@@ -9,6 +9,8 @@ Twitter Authentication, and miscellaneous methods that are useful when
 dealing with the Twitter API
 """
 
+import warnings
+
 import requests
 from requests.auth import HTTPBasicAuth
 from requests_oauthlib import OAuth1, OAuth2
@@ -20,36 +22,46 @@ from .endpoints import EndpointsMixin
 from .exceptions import TwythonError, TwythonAuthError, TwythonRateLimitError
 from .helpers import _transparent_params
 
-import warnings
-
 warnings.simplefilter('always', TwythonDeprecationWarning)  # For Python 2.7 >
 
 
 class Twython(EndpointsMixin, object):
     def __init__(self, app_key=None, app_secret=None, oauth_token=None,
-                 oauth_token_secret=None, access_token=None, token_type='bearer',
-                 oauth_version=1, api_version='1.1', client_args=None, auth_endpoint='authenticate'):
-        """Instantiates an instance of Twython. Takes optional parameters for authentication and such (see below).
+                 oauth_token_secret=None, access_token=None,
+                 token_type='bearer', oauth_version=1, api_version='1.1',
+                 client_args=None, auth_endpoint='authenticate'):
+        """Instantiates an instance of Twython. Takes optional parameters for
+        authentication and such (see below).
 
         :param app_key: (optional) Your applications key
         :param app_secret: (optional) Your applications secret key
-        :param oauth_token: (optional) When using **OAuth 1**, combined with oauth_token_secret to make authenticated calls
-        :param oauth_token_secret: (optional) When using **OAuth 1** combined with oauth_token to make authenticated calls
-        :param access_token: (optional) When using **OAuth 2**, provide a valid access token if you have one
-        :param token_type: (optional) When using **OAuth 2**, provide your token type. Default: bearer
-        :param oauth_version: (optional) Choose which OAuth version to use. Default: 1
-        :param api_version: (optional) Choose which Twitter API version to use. Default: 1.1
-
-        :param client_args: (optional) Accepts some requests Session parameters and some requests Request parameters.
-                            See http://docs.python-requests.org/en/latest/api/#sessionapi and requests section below it for details.
-                            [ex. headers, proxies, verify(SSL verification)]
-        :param auth_endpoint: (optional) Lets you select which authentication endpoint will use your application.
-                              This will allow the application to have DM access if the endpoint is 'authorize'.
-                              Default: authenticate.
+        :param oauth_token: (optional) When using **OAuth 1**, combined with
+        oauth_token_secret to make authenticated calls
+        :param oauth_token_secret: (optional) When using **OAuth 1** combined
+        with oauth_token to make authenticated calls
+        :param access_token: (optional) When using **OAuth 2**, provide a
+        valid access token if you have one
+        :param token_type: (optional) When using **OAuth 2**, provide your
+        token type. Default: bearer
+        :param oauth_version: (optional) Choose which OAuth version to use.
+        Default: 1
+        :param api_version: (optional) Choose which Twitter API version to
+        use. Default: 1.1
 
+        :param client_args: (optional) Accepts some requests Session parameters
+        and some requests Request parameters.
+              See http://docs.python-requests.org/en/latest/api/#sessionapi
+              and requests section below it for details.
+              [ex. headers, proxies, verify(SSL verification)]
+        :param auth_endpoint: (optional) Lets you select which authentication
+        endpoint will use your application.
+              This will allow the application to have DM access
+              if the endpoint is 'authorize'.
+                Default: authenticate.
         """
 
-        # API urls, OAuth urls and API version; needed for hitting that there API.
+        # API urls, OAuth urls and API version; needed for hitting that there
+        # API.
         self.api_version = api_version
         self.api_url = 'https://api.twitter.com/%s'
 
@@ -75,16 +87,18 @@ class Twython(EndpointsMixin, object):
 
         self.client_args = client_args or {}
         default_headers = {'User-Agent': 'Twython v' + __version__}
-        if not 'headers' in self.client_args:
+        if 'headers' not in self.client_args:
             # If they didn't set any headers, set our defaults for them
             self.client_args['headers'] = default_headers
         elif 'User-Agent' not in self.client_args['headers']:
-            # If they set headers, but didn't include User-Agent.. set it for them
+            # If they set headers, but didn't include User-Agent.. set
+            # it for them
             self.client_args['headers'].update(default_headers)
 
         # Generate OAuth authentication object for the request
         # If no keys/tokens are passed to __init__, auth=None allows for
-        # unauthenticated requests, although I think all v1.1 requests need auth
+        # unauthenticated requests, although I think all v1.1 requests
+        # need auth
         auth = None
         if oauth_version == 1:
             # User Authentication is through OAuth 1
@@ -93,12 +107,14 @@ class Twython(EndpointsMixin, object):
                 auth = OAuth1(self.app_key, self.app_secret)
 
             if self.app_key is not None and self.app_secret is not None and \
-               self.oauth_token is not None and self.oauth_token_secret is not None:
+               self.oauth_token is not None and self.oauth_token_secret is \
+               not None:
                 auth = OAuth1(self.app_key, self.app_secret,
                               self.oauth_token, self.oauth_token_secret)
         elif oauth_version == 2 and self.access_token:
             # Application Authentication is through OAuth 2
-            token = {'token_type': token_type, 'access_token': self.access_token}
+            token = {'token_type': token_type,
+                     'access_token': self.access_token}
             auth = OAuth2(self.app_key, token=token)
 
         self.client = requests.Session()
@@ -109,10 +125,14 @@ class Twython(EndpointsMixin, object):
         # Never be used again.
         client_args_copy = self.client_args.copy()
         for k, v in client_args_copy.items():
-            if k in ('cert', 'headers', 'hooks', 'max_redirects', 'proxies'):
+            if k in ('cert', 'hooks', 'max_redirects', 'proxies'):
                 setattr(self.client, k, v)
                 self.client_args.pop(k)  # Pop, pop!
 
+        # Headers are always present, so we unconditionally pop them and merge
+        # them into the session headers.
+        self.client.headers.update(self.client_args.pop('headers'))
+
         self._last_call = None
 
     def __repr__(self):
@@ -143,7 +163,6 @@ class Twython(EndpointsMixin, object):
             response = func(url, **requests_args)
         except requests.RequestException as e:
             raise TwythonError(str(e))
-        content = response.content.decode('utf-8')
 
         # create stash for last function intel
         self._last_call = {
@@ -153,75 +172,89 @@ class Twython(EndpointsMixin, object):
             'headers': response.headers,
             'status_code': response.status_code,
             'url': response.url,
-            'content': content,
+            'content': response.text,
         }
 
-        #  Wrap the json loads in a try, and defer an error
-        #  Twitter will return invalid json with an error code in the headers
-        json_error = False
-        try:
-            try:
-                # try to get json
-                content = content.json()
-            except AttributeError:
-                # if unicode detected
-                content = json.loads(content)
-        except ValueError:
-            json_error = True
-            content = {}
-
+        # greater than 304 (not modified) is an error
         if response.status_code > 304:
-            # If there is no error message, use a default.
-            errors = content.get('errors',
-                                 [{'message': 'An error occurred processing your request.'}])
-            if errors and isinstance(errors, list):
-                error_message = errors[0]['message']
-            else:
-                error_message = errors  # pragma: no cover
+            error_message = self._get_error_message(response)
             self._last_call['api_error'] = error_message
 
             ExceptionType = TwythonError
             if response.status_code == 429:
-                # Twitter API 1.1, always return 429 when rate limit is exceeded
-                ExceptionType = TwythonRateLimitError  # pragma: no cover
-            elif response.status_code == 401 or 'Bad Authentication data' in error_message:
+                # Twitter API 1.1, always return 429 when
+                # rate limit is exceeded
+                ExceptionType = TwythonRateLimitError
+            elif response.status_code == 401 or 'Bad Authentication data' \
+                    in error_message:
                 # Twitter API 1.1, returns a 401 Unauthorized or
-                # a 400 "Bad Authentication data" for invalid/expired app keys/user tokens
+                # a 400 "Bad Authentication data" for invalid/expired
+                # app keys/user tokens
                 ExceptionType = TwythonAuthError
 
-            raise ExceptionType(error_message,
-                                error_code=response.status_code,
-                                retry_after=response.headers.get('retry-after'))
+            raise ExceptionType(
+                error_message,
+                error_code=response.status_code,
+                retry_after=response.headers.get('X-Rate-Limit-Reset'))
 
-        # if we have a json error here, then it's not an official Twitter API error
-        if json_error and not response.status_code in (200, 201, 202):  # pragma: no cover
-            raise TwythonError('Response was not valid JSON, unable to decode.')
+        try:
+            content = response.json()
+        except ValueError:
+            raise TwythonError('Response was not valid JSON. \
+                               Unable to decode.')
 
         return content
 
+    def _get_error_message(self, response):
+        """Parse and return the first error message"""
+
+        error_message = 'An error occurred processing your request.'
+        try:
+            content = response.json()
+            # {"errors":[{"code":34,"message":"Sorry,
+            # that page does not exist"}]}
+            error_message = content['errors'][0]['message']
+        except ValueError:
+            # bad json data from Twitter for an error
+            pass
+        except (KeyError, IndexError):
+            # missing data so fallback to default message
+            pass
+
+        return error_message
+
     def request(self, endpoint, method='GET', params=None, version='1.1'):
         """Return dict of response received from Twitter's API
 
-        :param endpoint: (required) Full url or Twitter API endpoint (e.g. search/tweets)
+        :param endpoint: (required) Full url or Twitter API endpoint
+                         (e.g. search/tweets)
         :type endpoint: string
-        :param method: (optional) Method of accessing data, either GET or POST. (default GET)
+        :param method: (optional) Method of accessing data, either
+                       GET or POST. (default GET)
         :type method: string
-        :param params: (optional) Dict of parameters (if any) accepted the by Twitter API endpoint you are trying to access (default None)
+        :param params: (optional) Dict of parameters (if any) accepted
+                       the by Twitter API endpoint you are trying to
+                       access (default None)
         :type params: dict or None
-        :param version: (optional) Twitter API version to access (default 1.1)
+        :param version: (optional) Twitter API version to access
+                        (default 1.1)
         :type version: string
 
         :rtype: dict
         """
 
+        if endpoint.startswith('http://'):
+            raise TwythonError('api.twitter.com is restricted to SSL/TLS traffic.')
+
         # In case they want to pass a full Twitter URL
         # i.e. https://api.twitter.com/1.1/search/tweets.json
-        if endpoint.startswith('http://') or endpoint.startswith('https://'):
+        if endpoint.startswith('https://'):
             url = endpoint
         else:
             url = '%s/%s.json' % (self.api_url % version, endpoint)
 
-        content = self._request(url, method=method, params=params, api_call=url)
+        content = self._request(url, method=method, params=params,
+                                api_call=url)
 
         return content
 
@@ -237,7 +270,8 @@ class Twython(EndpointsMixin, object):
         """Returns a specific header from the last API call
         This will return None if the header is not present
 
-        :param header: (required) The name of the header you want to get the value of
+        :param header: (required) The name of the header you want to get
+                       the value of
 
         Most useful for the following header information:
             x-rate-limit-limit,
@@ -247,21 +281,31 @@ class Twython(EndpointsMixin, object):
 
         """
         if self._last_call is None:
-            raise TwythonError('This function must be called after an API call.  It delivers header information.')
+            raise TwythonError('This function must be called after an API call. \
+                               It delivers header information.')
 
         return self._last_call['headers'].get(header, default_return_value)
 
-    def get_authentication_tokens(self, callback_url=None, force_login=False, screen_name=''):
-        """Returns a dict including an authorization URL, ``auth_url``, to direct a user to
+    def get_authentication_tokens(self, callback_url=None, force_login=False,
+                                  screen_name=''):
+        """Returns a dict including an authorization URL, ``auth_url``, to
+           direct a user to
 
-        :param callback_url: (optional) Url the user is returned to after they authorize your app (web clients only)
-        :param force_login: (optional) Forces the user to enter their credentials to ensure the correct users account is authorized.
-        :param screen_name: (optional) If forced_login is set OR user is not currently logged in, Prefills the username input box of the OAuth login screen with the given value
+        :param callback_url: (optional) Url the user is returned to after
+                             they authorize your app (web clients only)
+        :param force_login: (optional) Forces the user to enter their
+                            credentials to ensure the correct users
+                            account is authorized.
+        :param screen_name: (optional) If forced_login is set OR user is
+                            not currently logged in, Prefills the username
+                            input box of the OAuth login screen with the
+                            given value
 
         :rtype: dict
         """
         if self.oauth_version != 1:
-            raise TwythonError('This method can only be called when your OAuth version is 1.0.')
+            raise TwythonError('This method can only be called when your \
+                               OAuth version is 1.0.')
 
         request_args = {}
         if callback_url:
@@ -269,15 +313,18 @@ class Twython(EndpointsMixin, object):
         response = self.client.get(self.request_token_url, params=request_args)
 
         if response.status_code == 401:
-            raise TwythonAuthError(response.content, error_code=response.status_code)
+            raise TwythonAuthError(response.content,
+                                   error_code=response.status_code)
         elif response.status_code != 200:
-            raise TwythonError(response.content, error_code=response.status_code)
+            raise TwythonError(response.content,
+                               error_code=response.status_code)
 
         request_tokens = dict(parse_qsl(response.content.decode('utf-8')))
         if not request_tokens:
             raise TwythonError('Unable to decode request tokens.')
 
-        oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed') == 'true'
+        oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed') \
+            == 'true'
 
         auth_url_params = {
             'oauth_token': request_tokens['oauth_token'],
@@ -293,21 +340,28 @@ class Twython(EndpointsMixin, object):
         if callback_url and not oauth_callback_confirmed:
             auth_url_params['oauth_callback'] = self.callback_url
 
-        request_tokens['auth_url'] = self.authenticate_url + '?' + urlencode(auth_url_params)
+        request_tokens['auth_url'] = self.authenticate_url + \
+            '?' + urlencode(auth_url_params)
 
         return request_tokens
 
     def get_authorized_tokens(self, oauth_verifier):
-        """Returns a dict of authorized tokens after they go through the :class:`get_authentication_tokens` phase.
+        """Returns a dict of authorized tokens after they go through the
+        :class:`get_authentication_tokens` phase.
 
-        :param oauth_verifier: (required) The oauth_verifier (or a.k.a PIN for non web apps) retrieved from the callback url querystring
+        :param oauth_verifier: (required) The oauth_verifier (or a.k.a PIN
+        for non web apps) retrieved from the callback url querystring
         :rtype: dict
 
         """
         if self.oauth_version != 1:
-            raise TwythonError('This method can only be called when your OAuth version is 1.0.')
+            raise TwythonError('This method can only be called when your \
+                               OAuth version is 1.0.')
 
-        response = self.client.get(self.access_token_url, params={'oauth_verifier': oauth_verifier}, headers={'Content-Type': 'application/json'})
+        response = self.client.get(self.access_token_url,
+                                   params={'oauth_verifier': oauth_verifier},
+                                   headers={'Content-Type': 'application/\
+                                   json'})
 
         if response.status_code == 401:
             try:
@@ -320,7 +374,8 @@ class Twython(EndpointsMixin, object):
             except ValueError:
                 content = {}
 
-            raise TwythonError(content.get('error', 'Invalid / expired Token'), error_code=response.status_code)
+            raise TwythonError(content.get('error', 'Invalid / expired To \
+            ken'), error_code=response.status_code)
 
         authorized_tokens = dict(parse_qsl(response.content.decode('utf-8')))
         if not authorized_tokens:
@@ -329,12 +384,14 @@ class Twython(EndpointsMixin, object):
         return authorized_tokens  # pragma: no cover
 
     def obtain_access_token(self):
-        """Returns an OAuth 2 access token to make OAuth 2 authenticated read-only calls.
+        """Returns an OAuth 2 access token to make OAuth 2 authenticated
+        read-only calls.
 
         :rtype: string
         """
         if self.oauth_version != 2:
-            raise TwythonError('This method can only be called when your OAuth version is 2.0.')
+            raise TwythonError('This method can only be called when your \
+                               OAuth version is 2.0.')
 
         data = {'grant_type': 'client_credentials'}
         basic_auth = HTTPBasicAuth(self.app_key, self.app_secret)
@@ -356,8 +413,10 @@ class Twython(EndpointsMixin, object):
     def construct_api_url(api_url, **params):
         """Construct a Twitter API url, encoded, with parameters
 
-        :param api_url: URL of the Twitter API endpoint you are attempting to construct
-        :param \*\*params: Parameters that are accepted by Twitter for the endpoint you're requesting
+        :param api_url: URL of the Twitter API endpoint you are attempting
+        to construct
+        :param \*\*params: Parameters that are accepted by Twitter for the
+        endpoint you're requesting
         :rtype: string
 
         Usage::
@@ -366,7 +425,8 @@ class Twython(EndpointsMixin, object):
           >>> twitter = Twython()
 
           >>> api_url = 'https://api.twitter.com/1.1/search/tweets.json'
-          >>> constructed_url = twitter.construct_api_url(api_url, q='python', result_type='popular')
+          >>> constructed_url = twitter.construct_api_url(api_url, q='python',
+          result_type='popular')
           >>> print constructed_url
           https://api.twitter.com/1.1/search/tweets.json?q=python&result_type=popular
 
@@ -382,23 +442,27 @@ class Twython(EndpointsMixin, object):
 
     def search_gen(self, search_query, **params):  # pragma: no cover
         warnings.warn(
-            'This method is deprecated. You should use Twython.cursor instead. [eg. Twython.cursor(Twython.search, q=\'your_query\')]',
+            'This method is deprecated. You should use Twython.cursor instead. \
+            [eg. Twython.cursor(Twython.search, q=\'your_query\')]',
             TwythonDeprecationWarning,
             stacklevel=2
         )
         return self.cursor(self.search, q=search_query, **params)
 
-    def cursor(self, function, **params):
+    def cursor(self, function, return_pages=False, **params):
         """Returns a generator for results that match a specified query.
 
-        :param function: Instance of a Twython function (Twython.get_home_timeline, Twython.search)
-        :param \*\*params: Extra parameters to send with your request (usually parameters excepted by the Twitter API endpoint)
+        :param function: Instance of a Twython function
+        (Twython.get_home_timeline, Twython.search)
+        :param \*\*params: Extra parameters to send with your request
+        (usually parameters accepted by the Twitter API endpoint)
         :rtype: generator
 
         Usage::
 
           >>> from twython import Twython
-          >>> twitter = Twython(APP_KEY, APP_SECRET, OAUTH_TOKEN, OAUTH_TOKEN_SECRET)
+          >>> twitter = Twython(APP_KEY, APP_SECRET, OAUTH_TOKEN,
+          OAUTH_TOKEN_SECRET)
 
           >>> results = twitter.cursor(twitter.search, q='python')
           >>> for result in results:
@@ -406,40 +470,45 @@ class Twython(EndpointsMixin, object):
 
         """
         if not hasattr(function, 'iter_mode'):
-            raise TwythonError('Unable to create generator for Twython method "%s"' % function.__name__)
+            raise TwythonError('Unable to create generator for Twython \
+                               method "%s"' % function.__name__)
 
-        content = function(**params)
+        while True:
+            content = function(**params)
 
-        if not content:
-            raise StopIteration
+            if not content:
+                raise StopIteration
 
-        if hasattr(function, 'iter_key'):
-            results = content.get(function.iter_key)
-        else:
-            results = content
+            if hasattr(function, 'iter_key'):
+                results = content.get(function.iter_key)
+            else:
+                results = content
 
-        for result in results:
-            yield result
+            if return_pages:
+                yield results
+            else:
+                for result in results:
+                    yield result
 
-        if function.iter_mode == 'cursor' and content['next_cursor_str'] == '0':
-            raise StopIteration
+            if function.iter_mode == 'cursor' and \
+               content['next_cursor_str'] == '0':
+                raise StopIteration
 
-        try:
-            if function.iter_mode == 'id':
-                if not 'max_id' in params:
-                    # Add 1 to the id because since_id and max_id are inclusive
-                    if hasattr(function, 'iter_metadata'):
-                        since_id = content[function.iter_metadata].get('since_id_str')
-                    else:
-                        since_id = content[0]['id_str']
-                    params['since_id'] = (int(since_id) - 1)
-            elif function.iter_mode == 'cursor':
-                params['cursor'] = content['next_cursor_str']
-        except (TypeError, ValueError):  # pragma: no cover
-            raise TwythonError('Unable to generate next page of search results, `page` is not a number.')
-
-        for result in self.cursor(function, **params):
-            yield result
+            try:
+                if function.iter_mode == 'id':
+                    if 'max_id' not in params:
+                        # Add 1 to the id because since_id and
+                        # max_id are inclusive
+                        if hasattr(function, 'iter_metadata'):
+                            since_id = content[function.iter_metadata].get('since_id_str')
+                        else:
+                            since_id = content[0]['id_str']
+                        params['since_id'] = (int(since_id) - 1)
+                elif function.iter_mode == 'cursor':
+                    params['cursor'] = content['next_cursor_str']
+            except (TypeError, ValueError):  # pragma: no cover
+                raise TwythonError('Unable to generate next page of search \
+                                   results, `page` is not a number.')
 
     @staticmethod
     def unicode2utf8(text):
@@ -461,11 +530,14 @@ class Twython(EndpointsMixin, object):
         """Return HTML for a tweet (urls, mentions, hashtags replaced with links)
 
         :param tweet: Tweet object from received from Twitter API
-        :param use_display_url: Use display URL to represent link (ex. google.com, github.com). Default: True
-        :param use_expanded_url: Use expanded URL to represent link (e.g. http://google.com). Default False
+        :param use_display_url: Use display URL to represent link
+        (ex. google.com, github.com). Default: True
+        :param use_expanded_url: Use expanded URL to represent link
+        (e.g. http://google.com). Default False
 
         If use_expanded_url is True, it overrides use_display_url.
-        If use_display_url and use_expanded_url is False, short url will be used (t.co/xxxxx)
+        If use_display_url and use_expanded_url is False, short url will
+        be used (t.co/xxxxx)
 
         """
         if 'retweeted_status' in tweet:
@@ -480,7 +552,8 @@ class Twython(EndpointsMixin, object):
                 start, end = entity['indices'][0], entity['indices'][1]
 
                 mention_html = '<a href="https://twitter.com/%(screen_name)s" class="twython-mention">@%(screen_name)s</a>'
-                text = text.replace(tweet['text'][start:end], mention_html % {'screen_name': entity['screen_name']})
+                text = text.replace(tweet['text'][start:end],
+                        mention_html % {'screen_name': entity['screen_name']})
 
             # Hashtags
             for entity in entities['hashtags']:
@@ -492,7 +565,8 @@ class Twython(EndpointsMixin, object):
             # Urls
             for entity in entities['urls']:
                 start, end = entity['indices'][0], entity['indices'][1]
-                if use_display_url and entity.get('display_url') and not use_expanded_url:
+                if use_display_url and entity.get('display_url') \
+                   and not use_expanded_url:
                     shown_url = entity['display_url']
                 elif use_expanded_url and entity.get('expanded_url'):
                     shown_url = entity['expanded_url']
@@ -500,6 +574,7 @@ class Twython(EndpointsMixin, object):
                     shown_url = entity['url']
 
                 url_html = '<a href="%s" class="twython-url">%s</a>'
-                text = text.replace(tweet['text'][start:end], url_html % (entity['url'], shown_url))
+                text = text.replace(tweet['text'][start:end],
+                                    url_html % (entity['url'], shown_url))
 
         return text
